/* artistMergeDialog.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "artistMergeDialog.h"
#include "format.h"
#include "artist.h"
#include "dataCell.h"
#include "database.h"

static const int num_clist_titles = 1;
static const gchar *
clist_titles[num_clist_titles] = { "Artist" };

ArtistMergeDialog::
ArtistMergeDialog(const Dataset &data,
		  int base_artist_id,
		  const MergeArtists &merge_artists) :
  PalomaDialog(data),
  _merge_artists(merge_artists),
  _clist(num_clist_titles, clist_titles),
  _ok_button("OK"),
  _cancel_button("Cancel")
{
  _data[TI_artists] = TableDataProxy(TI_artists, base_artist_id);
  layout_window();
  setup();
}

ArtistMergeDialog::
~ArtistMergeDialog() {
}

const ArtistMergeDialog::MergeArtists &ArtistMergeDialog::
get_merge_artists() const {
  return _merge_artists;
}

void ArtistMergeDialog::
ok_button_clicked() {
  do_merge();
  accepted();
  delete_self();
}

void ArtistMergeDialog::
cancel_button_clicked() {
  delete_self();
}

void ArtistMergeDialog::
layout_window() {
  set_title("Merge Artists");

  // The box for the main set of widgets.
  vbox_.pack_start(_box1);
  _box1.show();
  _box1.set_border_width(8);

  _line_1.set_text
    ("You are about to merge the following " +
     format_int(_merge_artists.size() + 1) + " artists.\n"
     "All references to any of the artists listed below will be\n"
     "replaced with a reference to an artist named:");

  _line_2.set_text(get_formatted_artist_name(_data[TI_artists]));
  _line_3.set_text("The original artist entries will be permanently removed.");

  _box1.pack_start(_line_1);
  _box1.pack_start(_line_2, true, true, 8);
  _box1.pack_start(_line_3);

  _line_1.show();
  _line_2.show();
  _line_3.show();

  // The list of artists.
  _artist_window.set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  _box1.pack_start(_artist_window, true, true, 8);
  _artist_window.show();

  _artist_window.add_with_viewport(_clist);

  _clist.column_titles_hide();
  _clist.set_selection_mode(GTK_SELECTION_MULTIPLE);
  _clist.show();

  const gchar *row_text[num_clist_titles] = {
    NULL,
  };

  string orig_artist_name = get_formatted_artist_name(_data[TI_artists]);
  row_text[0] = orig_artist_name.c_str();
  _clist.append(row_text);

  for (int t = 0; t < _merge_artists.size(); t++) {
    int artist_id = _merge_artists[t];

    TableDataProxy artist(TI_artists, artist_id);
    string artist_name = get_formatted_artist_name(artist);
    row_text[0] = artist_name.c_str();

    _clist.append(row_text);
  }

  _line_4.set_text
    ("This is a good thing to do when a given artist has accidentally been added\n"
     "to the database twice, for instance with two slightly different spellings.");

  _line_5.set_text("This operation cannot be reversed.");

  _line_6.set_text("Press OK to do this, or CANCEL not to do it.");

  _box1.pack_start(_line_4);
  _box1.pack_start(_line_5, true, true, 8);
  _box1.pack_start(_line_6);

  _line_4.show();
  _line_5.show();
  _line_6.show();

  // The OK button.
  _ok_button.set_usize(80, 36);
  action_area_.pack_start(_ok_button, true, false, 0);
  _ok_button.set_flags(GTK_CAN_DEFAULT);
  _ok_button.grab_default();
  _ok_button.show();
  connect_to_method(_ok_button.clicked, this, &ok_button_clicked);
  
  // The Cancel button.
  _cancel_button.set_usize(80, 36);
  action_area_.pack_start(_cancel_button, true, false, 0);
  _cancel_button.set_flags(GTK_CAN_DEFAULT);
  _cancel_button.show();
  connect_to_method(_cancel_button.clicked, this, &cancel_button_clicked);
}

void ArtistMergeDialog::
do_merge() {
  // Ok, the user has said "do it."  No turning back now!
  int keep_artist = _data[TI_artists].get_primary_key();
  assert(keep_artist >= 0);

  music_db->begin();

  MergeArtists::const_iterator mi;
  for (mi = _merge_artists.begin(); mi != _merge_artists.end(); ++mi) {
    int lose_artist = (*mi);
    music_db->sql_command
      ("UPDATE songs SET performer=" + DataCell(keep_artist) +
       " WHERE performer=" + DataCell(lose_artist));
    music_db->sql_command
      ("UPDATE songs SET writer=" + DataCell(keep_artist) +
       " WHERE writer=" + DataCell(lose_artist));
    music_db->sql_command
      ("UPDATE cds SET performer=" + DataCell(keep_artist) +
       " WHERE performer=" + DataCell(lose_artist));
    music_db->sql_command
      ("UPDATE cds SET writer=" + DataCell(keep_artist) +
       " WHERE writer=" + DataCell(lose_artist));
    music_db->sql_command
      ("DELETE FROM artists WHERE artist_id=" + DataCell(lose_artist));
  }

  music_db->commit();
}

